@@ -2,17 +2,25 @@
 import io
 import shutil
 import subprocess
+import sys
 import typing as tp
 from pathlib import Path
 
 from ._components import AllComponents
+from ._components import ExtraLevel
 from ._components._02_base import BaseComponent
+from ._components._03_meta import MetaMixin
+from ._components._98_sentinel import SentinelMixin
 from ._gitlab import GitlabSetter
+from ._proto_namespace import _ProtoNamespace
 from ._tick import TickSetter
+from ._todo import TodoSetter
 from ._util import LazyVersionStr
 
 
-class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
+class CliMixin(
+    SentinelMixin, MetaMixin, BaseComponent
+):  # pylint: disable=too-many-ancestors
     def __init__(self, target: Path, silent: bool = False) -> None:
         super().__init__()
         self.verbose = not silent
@@ -48,19 +56,16 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
                 break
         self._print(f"adaux {LazyVersionStr()} at your service", fg="blue")
 
-    @property
-    def target_custom(self) -> Path:
-        return self.target / "custom"
-
     def demo(self: "CliMixin") -> None:
-        self.clear_to_demo()
         out = io.StringIO()
-        self.save_auxcon_to_stream(out)
+        self.save_auxcon_to_stream(out, level=ExtraLevel.DEMODATA)
         self._print(out.getvalue())
 
     def show(self: "CliMixin", full: bool = False) -> None:
         out = io.StringIO()
-        self.save_auxcon_to_stream(out, stage1=not full)
+        self.save_auxcon_to_stream(
+            out, level=ExtraLevel.ENRICHED if full else ExtraLevel.DEFAULTED
+        )
         self._print(out.getvalue())
 
     def init(
@@ -71,18 +76,17 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
         author: str,
         force: bool = False,
     ) -> None:
-
-        self.clear_to_template(
-            project_name=project_name,
-            project_slug=project_slug,
-            python_version=python_version,
+        self.auxcon.project = _ProtoNamespace(
+            name=project_name,
+            slug=project_slug,
+            minimal_version=python_version,
             author=author,
         )
-
         self.target.mkdir(parents=True, exist_ok=True)
-        self.save_auxcon_and_print(force)
 
-    def save_auxcon_and_print(self, force: bool = False, stage1: bool = True) -> None:
+        self.save_auxcon_and_print(ExtraLevel.TEMPLATED, force=force)
+
+    def save_auxcon_and_print(self, level: ExtraLevel, force: bool = False) -> None:
         dest = self.auxcon_file
         over, fg_col = "", "green"
         if dest.exists():
@@ -90,28 +94,35 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
                 raise RuntimeError(f"{dest} exists, use -f or --force to overwrite.")
             over, fg_col = "over", "yellow"
 
-        with self.extra(stage1=stage1):
-            self.save_auxcon()
+        with self.extra(level=level) as aux:
+            self.save_auxcon(aux)
 
         # pylint: disable=protected-access
         self._print(f"{over}written template to {dest}", fg=fg_col)
 
     def sync(self: "CliMixin") -> None:
-        proj = self.auxcon.project
-        clean = self.type_wo_disabled(discard_before="SentinelMixin")()
+        with self.extra(ExtraLevel.FORMATTED) as aux:
+            proj = aux.project
+        clean: SentinelMixin = self.type_wo_disabled(discard_before="SentinelMixin")()  # type: ignore
         # full holds all possible defaults (in case something got disabled)
-        full = AllComponents().type_wo_disabled(discard_before="SentinelMixin")()  # type: ignore
+        full: SentinelMixin = AllComponents().type_wo_disabled(  # type: ignore
+            discard_before="SentinelMixin"
+        )()
 
         for ns in [clean, full]:
-            ns.clear_to_template(
-                project_name=proj.name,
-                project_slug=proj.slug,
-                python_version=proj.minimal_version,
+            # pylint: disable=no-member
+            ns.auxcon.project = _ProtoNamespace(
+                name=proj.name,
+                slug=proj.slug,
+                minimal_version=proj.minimal_version,
                 author=proj.author,
             )
-        self.update_to_template(clean.auxcon, full.auxcon)
-        with self.extra(stage1=True):
-            self.save_auxcon()
+
+        with self.extra(ExtraLevel.FORMATTED) as aux, clean.extra(
+            ExtraLevel.TEMPLATED
+        ), full.extra(ExtraLevel.TEMPLATED):
+            self.update_to_template(clean.auxf, full.auxf)
+            self.save_auxcon(aux)
 
     def bake(self: "CliMixin") -> None:
         if not self.auxcon_file.exists():
@@ -119,15 +130,15 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
         super().bake()
 
     def gitlab(self: "CliMixin", token: str) -> None:
-        gls = GitlabSetter(self, token)
         with self.extra():
+            gls = GitlabSetter(self, token)  # type: ignore
             gls.bake()
 
     def pipeline(
         self: "CliMixin", token: str, show_success: bool
     ) -> tp.Tuple[bool, bool]:
-        gls = GitlabSetter(self, token)
         with self.extra():
+            gls = GitlabSetter(self, token)  # type: ignore
             return gls.pipeline(show_success)
 
     def tick(
@@ -141,7 +152,7 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
             raise RuntimeError("cannot set major and minor tick at the same time!")
         if len(release_message) < 16:
             raise RuntimeError("message cannot be shorter than 16 char")
-        ticker = TickSetter(self, release_message, major, minor)
+        ticker = TickSetter(self, release_message, major, minor)  # type: ignore
         with self.extra():
             ticker.bake()
             if commit:
@@ -150,20 +161,20 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
     def release(
         self: "CliMixin", token: str, rerelease: bool = False
     ) -> tp.Tuple[bool, bool]:
-        gls = GitlabSetter(self, token)
         with self.extra():
+            gls = GitlabSetter(self, token)  # type: ignore
             return gls.release(rerelease)
 
     def convert(self: "CliMixin", src: Path, dest: Path, force: bool = False) -> None:
         self.auxcon_file = src
         self.load_auxcon()
         self.auxcon_file = dest
-        self.save_auxcon_and_print(force)
+        self.save_auxcon_and_print(ExtraLevel.DEFAULTED, force=force)
 
-    def ci(self: "CliMixin", *args: str) -> None:
+    def ci(self: "CliMixin", trigger_str: str, dry: bool = False) -> int:
         self._raise_if_disabled("ci")
-        cmd = ["python", str(self.target / "CI" / "python_ci.py"), *args]
-        subprocess.run(cmd, check=False)
+        with self.extra(ExtraLevel.HYDRATED), self.cwd_to_root():
+            return self.run_ci(trigger_str, dry=dry)  # type: ignore
 
     def cov(self: "CliMixin", open_html: bool = False) -> None:
         self._raise_if_disabled("coverage")
@@ -175,18 +186,21 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
                 datafile.unlink()
             else:
                 datafile.rename(cov_file)
-        cmd = [
-            "pytest",
-            "--cov",
-            f"--cov={self.auxcon.project.name}",
-            f"--cov-report=html:{cov_cache}",
-        ]
-        subprocess.run(cmd, check=True)
-        cov_file.rename(datafile)
 
-        if open_html:
-            cmd = ["open", str(cov_cache / "index.html")]
+        with self.extra(ExtraLevel.FORMATTED) as aux:
+            # pylint: disable=no-member
+            cmd = [
+                "pytest",
+                "--cov",
+                f"--cov={aux.project.name}",
+                f"--cov-report=html:{cov_cache}",
+            ]
             subprocess.run(cmd, check=True)
+            cov_file.rename(datafile)
+
+            if open_html:
+                cmd = ["open", str(cov_cache / "index.html")]
+                subprocess.run(cmd, check=True)
 
     def dcp(self: "CliMixin", *args: str) -> None:
         self._raise_if_disabled("docker")
@@ -208,14 +222,42 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
         ]
         subprocess.run(cmd, check=False)
 
+    def run(self: "CliMixin", *payload_names: str, force: bool = False) -> None:
+        self._raise_if_disabled("payload")
+        with self.extra(ExtraLevel.HYDRATED), self.cwd_to_root():
+            data = self.auxh.payload
+            if payload_names == ("ls",):
+                for title, flavor in [
+                    ("payloads", "docker_build"),
+                    ("with deps", "with_dependency"),
+                ]:
+                    list_ = list(data.lookup.level(flavor).values())
+                    if not list_:
+                        continue
+                    self._print("=" * len(title))
+                    self._print(title)
+                    self._print("=" * len(title))
+                    for payload in list_:
+                        print("-", payload.name)
+
+                return
+            try:
+                payloads = [data.lookup[x] for x in payload_names]
+            except KeyError as err:
+                raise RuntimeError(
+                    f"payload '{payload_names}' not found! Use aux run ls, or check the {self.auxcon_file.name} file."
+                ) from err
+            success = self.payload_run(*payloads, force=force)  # type: ignore
+            if not success:
+                sys.exit(1)
+
     def docs(self: "CliMixin", open_html: bool = False) -> None:
         self._raise_if_disabled("docs")
 
         docs_cache = self.target / "cache" / "docs"
 
-        with self.extra():
-            aux = self.auxcon
-
+        with self.extra() as aux:
+            # pylint: disable=no-member
             if aux.docs.framework == "sphinx":
                 cmd = ["sphinx-apidoc", "-o", aux.docs.root, aux.project.module_dir]
                 subprocess.run(cmd, check=True)
@@ -312,6 +354,27 @@ class CliMixin(BaseComponent):  # pylint: disable=too-many-ancestors
             for file in files:
                 file.unlink()
 
+    def todo_and_note(
+        self: "CliMixin",
+        flavor: str,
+        new: tp.Optional[str] = None,
+        close: tp.Optional[str] = None,
+        gitignore: bool = False,
+    ) -> None:
+        if new and close:
+            raise RuntimeError("cannot set new and close an the same time")
+
+        with self.extra(), self.cwd_to_root():
+            todo = TodoSetter(self)  # type: ignore
+            if gitignore:
+                todo.add_gitignore()
+            if new:
+                todo.new(new, flavor)
+            elif close:
+                todo.close(close, flavor)
+            else:
+                todo.show(flavor)
+
     def _raise_if_disabled(self, component: str) -> None:
         if not self.is_enabled(component):
             raise RuntimeError(
