@@ -4,6 +4,7 @@ import typing as tp
 
 from ..._proto_namespace import _ProtoNamespace
 from ._base import Payload
+from ._docker import DockerPayload
 
 __all__ = ["WithDependencyPayload"]
 
@@ -33,6 +34,12 @@ class WithDependencyPayload(Payload):
             dep.hydrate()
         self.payload.hydrate(self.deps)
 
+    @property
+    def executor(self) -> tp.Any:
+        if isinstance(self.payload, DockerPayload):
+            return self.payload.executor
+        raise NotImplementedError(self.payload.flavor)
+
     @property
     def param(self) -> _ProtoNamespace:
         # pylint: disable=no-member
