@@ -4,6 +4,8 @@ import typing as tp
 
 # py38 workaround
 if tp.TYPE_CHECKING:
+    # we use OrderedDict explicitly for readability
+    # and we use move_to_end
     OrderedDict = tp.OrderedDict[str, tp.Any]
 else:
     OrderedDict = collections.OrderedDict
@@ -29,3 +31,10 @@ class _ProtoNamespace(OrderedDict):
         for key, val in self.items():
             res.append(f"{repr(key)}: {repr(val)}")
         return "{" + ", ".join(res) + "}"
+
+    def insert_after(self, prev_key: str, key: str, val: tp.Any) -> None:
+        self[key] = val
+        keys = list(self.keys())
+        idx = keys.index(prev_key)
+        for move_key in keys[idx + 1 : -1]:
+            self.move_to_end(move_key)
